-- This is what we want parameterized
SELECT * FROM @DBName.dbo.@TableName

-- How do we get to this?
EXEC(N'SELECT * FROM ' + @DBName + N'.dbo.' + @TableName)

-- (1) Enter the string as if parameters were supported...
	SELECT * FROM @DBName.dbo.@TableName
-- (2) Add EXEC at the beginning and then a paren at both ends
	EXEC(SELECT * FROM @DBName.dbo.@TableName)
-- (3) Starting at the first paren add single quotes - right up to each variable!
	EXEC('SELECT * FROM '@DBName'.dbo.'@TableName)
-- (4) Make sure to add the + for concatenation
	EXEC('SELECT * FROM ' + @DBName + '.dbo.' + @TableName)
-- (5) Is it unicode? Preceed all strings with N
	EXEC(N'SELECT * FROM ' + @DBName + N'.dbo.' + @TableName)

-- How about a Proc?
CREATE PROC GetData
(
                @DBName            sysname,    
                @TableName        sysname
)
AS
DECLARE @ExecStr              nvarchar(4000)
SELECT @ExecStr = N'SELECT * FROM ' + @DBName + N'.dbo.' + @TableName
-- For Testing use the SELECT @ExecStr
--SELECT @ExecStr 
-- Once Tested use EXEC(@ExecStr) for execution
EXEC(@ExecStr)
go
EXEC GetData N'Northwind', N'Orders'
EXEC GetData N'pubs', N'authors'
EXEC GetData N'pubs', N'titles'
go

DECLARE @DBName            sysname,    --nvarchar(128)
                @TableName        sysname,
                @ExecStr              nvarchar(4000),
                @lname                varchar(30)
SELECT @DBName = N'pubs',
                @TableName = N'authors',
                @lname = QUOTENAME('O''leary', '''')
-- EXEC(N'SELECT * FROM ' + @DBName 
--             + N'..' + @TableName)
-- select * from authors
SELECT @ExecStr = N'SELECT * FROM ' + @DBName 
                                + N'.dbo.' + @TableName
                                + N' where au_lname = ' + @lname 
SELECT @ExecStr
EXEC(@ExecStr)
go

DBCC FREEPROCCACHE            --Clears all plans from cache
DBCC DROPCLEANBUFFERS    --Clears all data from cache
SELECT * FROM master..syscacheobjects
    WHERE cacheobjtype = 'Executable Plan'

SELECT * FROM dbo.titles WHERE price = 19.99
SELECT * FROM dbo.titles WHERE price = 199.99
SELECT * FROM dbo.titles WHERE price = $19.99

DECLARE @price    money,
                @ExecStr    nvarchar(4000)
SET @price = 19.99
SELECT @ExecStr = 'SELECT * FROM dbo.titles WHERE price = ' +
                                           convert(varchar(10), @price)
EXEC(@ExecStr)
-- or to force caching (caching by specification) 
-- use sp_executesql
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT * FROM dbo.titles WHERE price = @price'
EXEC sp_executesql @ExecStr,
                                    N'@price money',
                                    19.99

-- sp_executesql using multiple parameters
DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = 'SELECT * FROM dbo.titles WHERE price > @price AND type = @type'
EXEC sp_executesql @ExecStr,
                                    N'@price money, @type char(12)',
                                    10.00, 'business'

-- Sp_executesql within a dynamic string
DECLARE @tablename	sysname
SELECT @tablename = 'titles'
EXEC('DECLARE @ExecStr    nvarchar(4000)
SELECT @ExecStr = ''SELECT * FROM ' + @tablename + ' WHERE price = @price''
EXEC sp_executesql @ExecStr,
                                    N''@price money'',
                                    19.99')

-- Example using Output
DECLARE @tname	sysname,
		@DBUse		nvarchar(200),
		@StrResult	int,
		@ExecStr	nvarchar(2000)
SELECT @tname = 'authors'
SELECT @DBUse = 'Use pubs '
SELECT @ExecStr = (@DBUse + ' SELECT @StrResult=OBJECTPROPERTY(object_id(''' + QUOTENAME(@tname, ']') + '''),''IsTable'')')
EXECUTE sp_executesql @ExecStr, N'@StrResult int OUTPUT', @StrResult OUTPUT
SELECT @StrResult
SELECT @ExecStr = (@DBUse + ' SELECT @StrResult=object_id(''' + QUOTENAME(@tname, ']') + ''')')
EXECUTE sp_executesql @ExecStr, N'@StrResult int OUTPUT', @StrResult OUTPUT
SELECT @StrResult